! =================================================================================================================================
! MODULE           : stomate_resp
!
! CONTACT          : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE          : IPSL (2006)
!                  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF           Calculates maintenance respiration for different plant components
!!
!!\n DESCRIPTION   : None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	   :
!!- McCree KJ. An equation for the respiration of white clover plants grown under controlled conditions. 
!! In: Setlik I, editor. Prediction and measurement of photosynthetic productivity. Wageningen, The Netherlands: 
!! Pudoc; 1970. p. 221-229.
!! - Krinner G, Viovy N, de Noblet-Ducoudre N, Ogee J, Polcher J, Friedlingstein P,
!! Ciais P, Sitch S, Prentice I C (2005) A dynamic global vegetation model for studies
!! of the coupled atmosphere-biosphere system. Global Biogeochemical Cycles, 19, GB1015,
!! doi: 10.1029/2003GB002199.\n
!! Ruimy A., Dedieu G., Saugier B. (1996), TURC: A diagnostic model
!! of continental gross primary productivity and net primary productivity,
!! Global Biogeochemical Cycles, 10, 269-285.\n

!! SVN :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_resp.f90 $
!! $Date: 2014-02-13 19:02:34 +0100 (Thu, 13 Feb 2014) $
!! $Revision: 1905 $
!! \n
!_ ================================================================================================================================
 
MODULE stomate_resp

  ! modules used:
  USE stomate_data
  USE pft_parameters
  USE constantes  
  USE constantes_soil
  USE function_library, ONLY : biomass_to_lai

  IMPLICIT NONE

  ! private & public routines
  PRIVATE
  PUBLIC maint_respiration,maint_respiration_clear

  LOGICAL, SAVE                                              :: firstcall = .TRUE.                 !! first call
!$OMP THREADPRIVATE(firstcall)

  CONTAINS


!! ================================================================================================================================
!! SUBROUTINE 	: maint_respiration_clear
!!
!>\BRIEF        : Set the flag ::firstcall to .TRUE. and as such activate section 
!!                1.1 of the subroutine maint_respiration (see below).
!_ ================================================================================================================================

  SUBROUTINE maint_respiration_clear
    firstcall=.TRUE.
  END SUBROUTINE maint_respiration_clear


!! ================================================================================================================================
!! SUBROUTINE 	: maint_respiration
!!
!>\BRIEF         Calculate PFT maintenance respiration of each living plant part by 
!! multiplying the biomass of plant part by maintenance respiration coefficient which
!! depends on long term mean annual temperature. PFT maintenance respiration is carbon flux 
!! with the units @tex $(gC.m^{-2}dt^{-1})$ @endtex, and the convention is from plants to the 
!! atmosphere.
!!
!! DESCRIPTION : The maintenance respiration of each plant part for each PFT is the biomass of the plant 
!! part multiplied by maintenance respiration coefficient. The biomass allocation to different 
!! plant parts is done in routine stomate_alloc.f90. The maintenance respiration coefficient is 
!! calculated in this routine.\n
!!
!! The maintenance respiration coefficient is the fraction of biomass that is lost during 
!! each time step, which increases linearly with temperature (2-meter air temperature for aboveground plant
!! tissues; root-zone temperature for below-ground tissues). Air temperature is an input forcing variable. 
!! Root-zone temperature is a convolution of root and soil temperature profiles and also calculated 
!! in this routine.\n
!!
!! The calculation of maintenance respiration coefficient (fraction of biomass respired) depends linearly
!! on temperature:
!! - the relevant temperature for different plant parts (air temperature or root-zone temperature)\n
!! - intercept: prescribed maintenance respiration coefficients at 0 Degree Celsius for 
!!   different plant parts for each PFT in routine stomate_constants.f90\n
!! - slope: calculated with a quadratic polynomial with the multi-annual mean air temperature 
!! (the constants are in routine stomate_constants.f90) as follows\n 
!!    \latexonly
!!      \input{resp3.tex} 
!!    \endlatexonly
!!   Where, maint_resp_slope1, maint_resp_slope2, maint_resp_slope3 are constant in stomate_constants.f90.
!!   Then coeff_maint is calculated as follows:\n
!!    \latexonly
!!      \input{resp4.tex} 
!!    \endlatexonly  
!! If the calculation result is negative, maintenance respiration coefficient will take the value 0.
!! Therefore the maintenance respiration will also be 0.\n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): PFT maintenance respiration of different plant parts (::resp_maint_part_radia)
!!
!! REFERENCE(S)	:
!! McCree KJ. An equation for the respiration of white clover plants grown under controlled conditions. In: 
!! Setlik I, editor. Prediction and measurement of photosynthetic productivity. Wageningen, 
!! The Netherlands: Pudoc; 1970. p. 221-229.
!! Krinner G, Viovy N, de Noblet-Ducoudre N, Ogee J, Polcher J, Friedlingstein P,
!! Ciais P, Sitch S, Prentice I C (2005) A dynamic global vegetation model for studies
!! of the coupled atmosphere-biosphere system. Global Biogeochemical Cycles, 19, GB1015,
!! doi: 10.1029/2003GB002199.\n
!! Ruimy A., Dedieu G., Saugier B. (1996), TURC: A diagnostic model
!! of continental gross primary productivity and net primary productivity,
!! Global Biogeochemical Cycles, 10, 269-285.\n
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE maint_respiration (npts, dt, t2m, tlong_ref, &
       stempdiag, gpp, gpp_week, lab_fac, &
       veget_max, rprof, biomass, resp_maint_part_radia, &
       circ_class_n, circ_class_biomass)

!! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                      :: npts         !! Domain size - number of grid cells (unitless)
    REAL(r_std), INTENT(in)                         :: dt           !! Time step of the simulations (seconds)
    REAL(r_std), DIMENSION(:), INTENT(in)           :: t2m          !! 2 meter air temperature - forcing variable (K)
    REAL(r_std), DIMENSION(:), INTENT(in)           :: tlong_ref    !! Long term annual mean 2 meter reference air temperatures 
                                                                    !! calculated in stomate_season.f90 (K)
    REAL(r_std), DIMENSION(:,:), INTENT (in)        :: stempdiag    !! Soil temperature of each soil layer (K)
    REAL(r_std), DIMENSION(:,:), INTENT(in)         :: gpp          !! Gross primary production per day per unit PFT
                                                                    !! @tex $ (gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)         :: gpp_week     !! Weekly mean gross primary production per day per unit PFT
                                                                    !! @tex $ (gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)         :: veget_max    !! PFT "maximal" coverage fraction of a PFT (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)         :: rprof        !! PFT root depth as calculated in stomate.f90 from parameter 
                                                                    !! humcste which is root profile for different PFTs 
                                                                    !! in slowproc.f90 (m)
    REAL(r_std), DIMENSION(:,:), INTENT(in)         :: lab_fac      !! Activity of labile pool (??UNITS??)
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)       :: circ_class_n !! Number of individuals in each circ class
                                                                    !! @tex $(ind m^{-2})$ @endtex

    !! 0.2 Output variables

    REAL(r_std), DIMENSION(:,:,:), INTENT(out)      :: resp_maint_part_radia !! PFT maintenance respiration of different
                                                                    !! plant parts @tex $(gC.m^{-2}dt^{-1} )$ @endtex

    !! 0.3 Modified variables
    
    REAL(r_std), DIMENSION(:,:,:,:),  INTENT(inout) :: biomass      !! PFT total biomass calculated in stomate_alloc.f90 
                                                                    !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout):: circ_class_biomass !! Biomass components of the model tree  
                                                                    !! within a circumference class
                                                                    !! class @tex $(g C ind^{-1})$ @endtex
    !! 0.4 Local variables

    REAL(r_std), DIMENSION(npts,nvm)        :: lai                  !! PFT leaf area index @tex $(m^2 m^{-2})$ @endtex
    INTEGER(i_std)                          :: ipts,ivm,ipar,ibdl   !! Indeces (unitless)

!    REAL(r_std), SAVE, DIMENSION(0:nbdl)    :: z_soil               !! Variable to store depth of the different soil layers (m)

    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)                :: z_soil !! Soil levels (m)
!$OMP THREADPRIVATE(z_soil)

    REAL(r_std), DIMENSION(npts,nvm)        :: t_root               !! PFT root temperature (convolution of root and soil 
                                                                    !! temperature profiles) (K)
    REAL(r_std), DIMENSION(npts,nvm,nparts) :: coeff_maint          !! PFT maintenance respiration coefficients of different 
                                                                    !! plant compartments at 0 deg C 
                                                                    !! @tex $(g.g^{-1}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nparts)     :: t_maint_radia        !! Temperature which is pertinent for maintenance respiration, 
                                                                    !! which is air/root temperature for above/below-ground 
                                                                    !! compartments (K) 
    REAL(r_std), DIMENSION(nvm,nparts)      :: fcn                  !! C/N ratio in tissue (unitless)
    REAL(r_std), DIMENSION(npts,nvm)        :: resp_maint_demand    !! Rate of maintanance respiration
                                                                    !! @tex $(gC.dt^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)        :: resp_maint_supply    !! Rate of growth respiration
                                                                    !! @tex $(gC.dt^{-1})$ @endtex                                  
    REAL(r_std), DIMENSION(npts,nvm)        :: resp_maint           !! Effective rate of maintanance respiration
                                                                    !! @tex $(gC.dt^{-1})$ @endtex   
    REAL(r_std), DIMENSION(npts)            :: gtemp                !! Temperature response of respiration in the 
                                                                    !! Lloyd-Taylor Model (K)
    REAL(r_std), DIMENSION(npts)            :: tl                   !! Long term reference temperature in degrees Celcius 
                                                                    !! (= tlong_ref - 273.15) (C)
    REAL(r_std), DIMENSION(npts)            :: slope                !! slope of the temperature dependence of maintenance 
                                                                    !! respiration coefficient (1/K)
    REAL(r_std), DIMENSION(npts)            :: rpc                  !! Scaling factor for integrating vertical soil 
                                                                    !! profiles (unitless)
    REAL(r_std), DIMENSION(nvm)             :: coeff_maint_temp     !! PFT maintenance respiration coefficients of different 
                                                                    !! plant compartments at 0 deg C 
                                                                    !! @tex $(g.g^{-1}dt^{-1})$ @endtex
    REAL(r_std)                             :: deficit              !! Calculate maintenance respiration based on 
                                                                    !! tissue pools (or labile pool size) (??UNIT??)
    REAL(r_std)                             :: limit_cn             !! Calculate maintenance respiration based on 
                                                                    !! tissue pools (or labile pool size) (??UNIT??)
    REAL(r_std)                             :: temp_share           !! Temporary variable to store the share
                                                                    !! of biomass of each circumference class
                                                                    !! to the total biomass            
    REAL(r_std)                             :: temp_class_biomass   !! Biomass across parts for a single circ
                                                                    !! class @tex $(gC m^{-2})$ @endtex
    REAL(r_std)                             :: temp_total_biomass   !! Biomass across parts and circ classes
                                                                    !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std)                             :: temp                 !! generic temporary variable 

   !! 0.5 Externalize
   REAL(r_std), PARAMETER                   :: gtemp_ref=1.6        !! Correction factor for respiration calculation, 
                                                                    !! since ORCHIDEE maintenance respiration rates
                                                                    !! refer to zero degrees but Lloyd and Taylor assumes 
                                                                    !! base temperature of 10 degrees (unitless)
! tzjwb
   INTEGER(i_std)                                              :: ier !! Error handling
       
!_ ================================================================================================================================
    
    
    IF (bavard.GE.4) WRITE(numout,*) 'Entering maintenance respiration'
   
 !! 1. Initializations
    
    IF ( firstcall ) THEN

       !! 1.1. Soil levels (first call only)
       !       Set the depth of the different soil layers (number of layers: nbdl) 
       !       previously calculated as variable diaglev in routines sechiba.f90 and slowproc.f90

! tzjwb
       ALLOCATE(z_soil(0:nbdl), stat=ier)
       IF ( ier /= 0 ) CALL ipslerr_p(3,'stomate_resp','Pb in allocate of z_soil','','')

       z_soil(0) = zero
       z_soil(1:nbdl) = diaglev(1:nbdl)

       ! Set first call to false for all subsequent calls 
       firstcall = .FALSE.

    ENDIF

    
    !! 1.2. Calculate root temperature
    !       Calculate root temperature as the convolution of root and soil temperature profiles
    DO ivm = 2,nvm ! Loop over # PFTs

       !! 1.2.1 Calculate rpc
       !  - rpc is an integration constant to make the integral over the root profile is equal 'one', 
       !    calculated as follows:\n
       !  \latexonly
       !    \input{resp1.tex} 
       !  \endlatexonly
       rpc(:) = un / ( un - EXP( -z_soil(nbdl) / rprof(:,ivm) ) )

       !! 1.2.2 Calculate root temperature
       !        - Integrate root profile temperature (K) over soil layers (number of layers = nbdl)
       !          with rpc and soil temperature (K) of each soil layer as follows:\n
       !        \latexonly
       !          \input{resp2.tex} 
       !        \endlatexonly
       !        Where, stempdiag is diagnostic temperature profile of soil (K)\n
       t_root(:,ivm) = zero

       DO ibdl = 1, nbdl ! Loop over # soil layers

          t_root(:,ivm) = &
               t_root(:,ivm) + stempdiag(:,ibdl) * rpc(:) * &
               ( EXP( -z_soil(ibdl-1)/rprof(:,ivm) ) - EXP( -z_soil(ibdl)/rprof(:,ivm) ) )

       ENDDO ! Loop over # soil layers

    ENDDO ! Loop over # PFTs

 !! 2. Define maintenance respiration coefficients

    DO ivm = 2,nvm ! Loop over # PFTs

       !! 2.1 Temperature for maintenanace respiration
       !      Temperature which is used to calculate maintenance respiration for different plant compartments
       !      (above- and belowground)\n
       !      - for aboveground parts, we use 2-meter air temperature, t2m\n
       !      - for belowground parts, we use root temperature calculated in section 1.2 of this subroutine\n
       
       ! 2.1.1 Aboveground biomass
       t_maint_radia(:,ileaf) = t2m(:)
       t_maint_radia(:,isapabove) = t2m(:)
       t_maint_radia(:,ifruit) = t2m(:)

       ! 2.1.2 Belowground biomass
       t_maint_radia(:,isapbelow) = t_root(:,ivm)
       t_maint_radia(:,iroot) = t_root(:,ivm)

       !! 2.1.3 Heartwood biomass
       !  Heartwood does does not respire (coeff_maint_zero is set to zero). 
       !  Any temperature could have been set  
       !  [code cleaning: set t(heartwood) to undef to 'undef']

       t_maint_radia(:,iheartbelow) = t_root(:,ivm)
       t_maint_radia(:,iheartabove) = t2m(:)

       !! 2.1.4 Reserve biomass
       !  Use aboveground temperature for trees and belowground temeperature for grasses
       IF ( is_tree(ivm) ) THEN

          t_maint_radia(:,icarbres) = t2m(:)

       ELSE

          t_maint_radia(:,icarbres) = t_root(:,ivm)

       ENDIF

       !! 2.1.5 Labile biomass pool
       !  Use aboveground temperature for the labile biomass pool
       t_maint_radia(:,ilabile) = t2m(:)
       
       !! 2.2 Calculate maintenance respiration coefficients (coeff_maint)
       !  Maintenance respiration is a fraction of biomass defined by the coefficient 
       !  coeff_maint [Mc Cree, 1969]. Coeff_maint is defined through a linear relationship of temperature [Ruimy et al, 1996]
       !  which slope is the coefficient 'slope' and which intercept is 'coeff_maint_zero'.
       !  - Coeff_maint_zero is defined in stomate_data to cm_zero_plantpartname
       !  - Slope is calculated here through a second-degree polynomial [Krinner et al, 2005] 
       !  equation that makes it dependent on the long term temperature (to represent adaptation
       !  of the ecosystem to long term temperature).
       !   \latexonly
       !    \input{resp3.tex} 
       !   \endlatexonly
       !  Where, maint_resp_slope1, maint_resp_slope2, maint_resp_slope3 are constant in stomate_constants.f90.
       !  Then coeff_maint is calculated as follows:\n
       !   \latexonly
       !    \input{resp4.tex} 
       !   \endlatexonly
       !  If the calculation result is negative, coeff_maint will take the value 0.\n	
       tl(:) = tlong_ref(:) - ZeroCelsius
       slope(:) = maint_resp_slope(ivm,1) + tl(:) * maint_resp_slope(ivm,2) + &
            tl(:)*tl(:) * maint_resp_slope(ivm,3)  !maint_resp_slope(ivm,1)=0.20

       DO ipar = 1, nparts ! Loop over # plant parts

          ! Resource-based allocation       
          IF ( .NOT.control%ok_functional_allocation ) THEN

            coeff_maint(:,ivm,ipar) = MAX( (coeff_maint_zero(ivm,ipar)*dt/one_day) * &
                 ( un + slope(:) * (t_maint_radia(:,ipar)-ZeroCelsius) ), zero ) 

         ! Allometric-based allocation    
         ELSE

            !+++CHECK+++
            ! The original code refers to the Sitch et al 2003 as the source of the equations and parameters 
            ! for modelling maintenance respiration. The equations below are consistent with the paper. The 
            ! parameter setting for coeff_maint is in the range of 0.066 to 0.011 as reported in the paper but
            ! exact values are not given. Although, the principle of a climate correction for coeff_maint is 
            ! mentioned in Sitch et al 2003, the reduction factors themselves were not given. As it appears 
            ! now this block of code pretends much more knowledge then we actually have. Rather than using a 
            ! baseline coeff_maint that is later corrected for the climate region, the parameter values for 
            ! coeff_maint could be simply prescribed and made pft-specific.
            ! Further down in the code C/N ratios are used to constrain respiration. The C/N ratios were reset 
            ! to the values presented in Sitch et al 2003 but still seem on the low side. If pft-specific 
            ! values are to be used, changes in respiration could be compensated for by changing 
            ! coeff_maint_init. Given Vicca et al 2012 (Ecology Letters) an NPP/GPP ratio of 0.5 is 'universal' 
            ! for forests given a sufficient nutrient supply and strictly defining NPP as solely its biomass 
            ! components (thus excluding VOC, exudation and subsidies to myccorrhizae as is the case in ORCHIDEE).
            ! Unless observation based values are available for coeff_maint_init, these values could be adjusted
            ! within the range of 0.066 to 0.011 to obtain an NPP/GPP of 0.5 in the absence of nutrient 
            ! limitations. 
               
            ! LPJ respiration factors based on Sitch et al. 2003 - first part of the calculation
            IF ( ipar.EQ.ileaf .OR. ipar.EQ.iroot .OR. ipar.EQ.ifruit .OR. &
                 ipar.EQ.isapabove .OR. ipar.EQ.isapbelow .OR. ipar.EQ.ilabile) THEN
               
               ! Sonke uses
               !coeff_maint(:,ivm,ipar) = 0.0548

               ! The Sitch et al 2003 paper gives a value of 0.066 (Table 3) and 0.011 for tropical systems
               !coeff_maint(:,ivm,ipar) = 0.066
 
               ! Use an effective PFT-specific value - this value has been optimized
               ! to reproduce observed NPP/GPP ratios
               coeff_maint(:,ivm,ipar) = coeff_maint_init(ivm)

            ! heartwood and reserves pool   
            ELSE

               coeff_maint(:,ivm,ipar) = zero

            ENDIF

            !+++BEGIN+DELETE+++
            ! Correction for climate region.
            ! [remove hard coding]
            ! Respiration rates for tropical forests
            !IF ( ivm.LE.3 ) THEN
               
               ! Sonke uses
               ! coeff_maint(:,ivm,ipar) = coeff_maint(:,ivm,ipar)*0.33
     
               ! The Sitch et al 2003 paper
               !coeff_maint(:,ivm,ipar) = coeff_maint(:,ivm,ipar)/6.

            ! Respiration rates for tropical grass and crops
            !ELSEIF ( ivm.EQ.11 .OR. ivm.EQ.13 ) THEN
               
               ! Sonke uses
               !coeff_maint(:,ivm,ipar) = coeff_maint(:,ivm,ipar)*0.5
               
               ! The Sitch et al 2003 paper
               !coeff_maint(:,ivm,ipar) = coeff_maint(:,ivm,ipar) / 6.
 
            ! Respiration rates for boreal and temperate PFTs
            !ELSE

               ! Sonke uses 
               ! coeff_maint(:,ivm,ipar) = coeff_maint(:,ivm,ipar)*0.7
               
               ! The Sitch et al 2003 paper - value should not be adjusted
               !coeff_maint(:,ivm,ipar) = coeff_maint(:,ivm,ipar)
               !coeff_maint(:,ivm,ipar) = coeff_maint(:,ivm,ipar)*0.5

            !ENDIF
            !+++END+DELETE+++

         ENDIF ! .NOT.control%ok_functional_allocation

       ENDDO ! Loop over # plant parts
     
        
       !+++OCN-code+++
!$       ! Calculate lai
!$       lai(:,ivm) = biomass(:,ivm,ileaf,icarbon) * sla(ivm)

!$       ! Leaf respiration for photosynthesis according to Friend et al. as in Zaehle et al.
!$       ! Leaf respiration is calculated in diffuco.f90
!$       IF ( control%ok_cexchange ) THEN
!$          coeff_maint_leaf(:,ivm,:) = zero
       
!$       ! Leaf respiration for the standard ORCHIDEE photosynthesis
!$       ELSE
!$          WHERE ( (biomass(:,ivm,ileaf,icarbon) .GT. min_stomate) .AND. (lai(:,ivm) .GT. min_stomate) ) 
!$           
!$             coeff_maint_leaf(:,ivm,1) = coeff_maint_zero(ivm,ileaf) * biomass(:,ivm,ileaf,icarbon) * & 
!$                  ( maint_resp_min_vmax*lai(:,ivm) + maint_resp_coeff*(un-exp(-ext_coeff(ivm)*lai(:,ivm))) ) &
!$                  / lai(:,ivm) / one_day
!$           
!$             coeff_maint_leaf(:,ivm,2) = slope(:)
!$           
!$          ELSEWHERE
             
!$              coeff_maint_leaf(:,ivm,1) = zero
!$              coeff_maint_leaf(:,ivm,2) = zero
             
!$          ENDWHERE
         
!$       ENDIF ! control%ok_cexchange
       !++++++++++

    ENDDO ! Loop over # PFTs
       

 !! 3. Calculate maintenance respiration

    ! Resource-based allocation
    IF ( .NOT.control%ok_functional_allocation ) THEN

       ! The maintenance respiration @tex $(gC.m^{-2}dt^{-1})$ @endtex of each plant compartment for each PFT is 
       ! the biomass @tex $(gC.m^{-2})$ @endtex of the plant part multiplied by maintenance respiration 
       ! coefficient @tex $(g.g^{-1}dt^{-1})$ @endtex, except that the maintenance respiration of leaves is 
       ! corrected by leaf area index (LAI) as follows:\n
       ! \latexonly     
       !   \input{resp5.tex} 
       ! \endlatexonly
    
       ! ibare_sechiba = 1, which means the there is only bare soil but not any PFT, consequently no LAI and
       ! no  maintenance respiration
       lai(:,ibare_sechiba) = zero
       resp_maint_part_radia(:,ibare_sechiba,:) = zero
    
       ! 3.1 Maintenance respiration of the different plant parts
       DO ivm = 2,nvm ! Loop over # PFTs
          
          ! Calculate lai
          DO ipts = 1,npts

             lai(ipts,ivm) = biomass_to_lai(biomass(ipts,ivm,ileaf,icarbon),ivm)

          ENDDO

       DO ipar = 1, nparts ! Loop over # plant parts

             ! Leaf respiration: depends on leaf mass and LAI 
             IF ( ipar .EQ. ileaf ) THEN

                WHERE ( (biomass(:,ivm,ileaf,icarbon).GT.min_stomate) .AND. (lai(:,ivm).GT.min_stomate) )

                   ! Maintenance respiration is calculated as a fraction of biomass as defined by coeff_maint 
                   ! and is adjusted for the nitrogen effect through a third factor depending on LAI. The 
                   ! hypothesis here is that the vcmax (i.e. the nitrogen distribution) in the canopy decreases 
                   ! exponentially with LAI following the Beer-Lambert law with an asymptote defining the 
                   ! minimum of the function at 30% of the LAI. The 1.4 parameter is an integration constant.
                   ! This method is also used in diffuco_trans_co2 2.4.1 for scaling vmax based on nitrogen 
                   ! reduction in the canopy.
                   resp_maint_part_radia(:,ivm,ipar) = coeff_maint(:,ivm,ipar) * biomass(:,ivm,ipar,icarbon) * &
                        ( maint_resp_min_vmax*lai(:,ivm) + maint_resp_coeff*(un - exp(-ext_coeff(ivm)*lai(:,ivm))) ) / lai(:,ivm)

                ELSEWHERE

                   resp_maint_part_radia(:,ivm,ileaf) = zero

                ENDWHERE

             ! All other components
             ELSE

                ! All components except ileaves and ilabile
                IF ( ipar.NE.ilabile ) THEN

                   resp_maint_part_radia(:,ivm,ipar) = coeff_maint(:,ivm,ipar) * biomass(:,ivm,ipar,icarbon)

                ! Respiration for ilabile
                ELSE

                   resp_maint_part_radia(:,ivm,ilabile) = zero

                ENDIF !ipar.NE.ilabile

             ENDIF !ipar.EQ.ileaf

          ENDDO ! Loop over # plant parts

       ENDDO ! Loop over # PFTs

    ! Allometric-based allocation
    ELSE
!WRITE(*,*) "tzjeval allom" 
       ! In the allometric based allocation scheme, maintentance respiration is calculated 
       ! the minimum of supply and demand
       resp_maint_part_radia(:,:,:) = zero
      
       DO ivm = 2,nvm ! Loop over # PFTs

          ! Demand based respiration
          resp_maint_demand(:,ivm) = 0.0

          DO ipar = 1, nparts ! Loop over # plant parts  
            
             ! LPJ respiration factors based on Sitch et al. 2003 - second part of the calculation
             ! Temperature response, LLoyd and Taylor, 1994. E0 = 308.56 comes from the paper of 
             ! Lloyd and Taylor but was fitted for soil respiration which only partly consists of
             ! authotrophic (root) respiration.   
             WHERE(t_maint_radia(:,ipar)-ZeroCelsius.GT.-46.01)

                gtemp(:) = dt/one_day * & 
                    EXP(308.56*(1.0/56.02-1.0/(t_maint_radia(:,ipar)-ZeroCelsius+46.02)))

             ! No gtemp below -46.01 degrees Celsius
             ELSEWHERE

                gtemp(:) = 0.0

             ENDWHERE           
            
             ! Calculate the limiting C/N ratio
             IF ( ipar.EQ.ileaf ) THEN

                limit_cn = cn_leaf_prescribed(ivm)

             ELSEIF ( ipar.EQ.iroot .OR. ipar.EQ.ifruit ) THEN

                limit_cn = cn_leaf_prescribed(ivm)/fcn_root(ivm)

             ELSEIF ( ipar.EQ.isapabove .OR. ipar.EQ.isapbelow ) THEN

                limit_cn = cn_leaf_prescribed(ivm)/fcn_wood(ivm)

             ELSE

                limit_cn = cn_leaf_prescribed(ivm)

             ENDIF
            
             !---TEMP---
             !IF (ivm.EQ.test_pft) THEN  
             !   WRITE(numout,*) 'limit - ipar, temp output,', ipar, limit_cn
             !ENDIF
             !----------

             resp_maint_part_radia(:,ivm,ipar) = coeff_maint(:,ivm,ipar) * gtemp(:) * & 
                    biomass(:,ivm,ipar,icarbon) / limit_cn
             resp_maint_demand(:,ivm) = resp_maint_demand(:,ivm) + resp_maint_part_radia(:,ivm,ipar)

          ENDDO ! Loop over # plant parts

       ENDDO ! Loop over # PFTs
   
    ENDIF ! .NOT.control%ok_functional_allocation

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving maintenance respiration'

  END SUBROUTINE maint_respiration

END MODULE stomate_resp
